PrecacheSound("respawn.wav")

vo_death_sounds <- ["vo/mercenary_paincrticialdeath01.mp3", "vo/mercenary_paincrticialdeath02.mp3", "vo/mercenary_paincrticialdeath03.mp3", "vo/mercenary_paincrticialdeath04.mp3"];
foreach (sound in vo_death_sounds)
    PrecacheSound(sound);

self.AddHudHideFlags(HIDEHUD_MISCSTATUS | HIDEHUD_METAL);

if (redTeamOnly && self.GetTeam() != TF_TEAM_RED)
{
    self.SwitchTeam(TF_TEAM_RED);
    self.ForceRespawn();
}

//OnSelfEvent("player_spawn_post", 0, function(player)
OnSelfEvent("post_inventory_application", 0, function()
{
    if (!IsValidRound())
        return;
    SetPropInt(self, "m_Shared.m_bShieldEquipped", 0);
    //Equip new weapons
    EquipWeapon(GetWeaponTypeByName("crowbar"));
    //self.Weapon_Switch(EquipWeapon(GetCurrentGunGameWeapon(self)));

    //Refresh model, movespeed and HP
    RefreshCharacter(false);

    //Add Merc's attributes
    self.AddCustomAttribute("cancel falling damage", 1, -1);
    self.AddCustomAttribute("crit mod disabled hidden", 0, -1);
    self.AddCustomAttribute("voice pitch scale", 0, -1);
    self.AddCustomAttribute("no double jump", 1, -1);
    self.AddCond(TF_COND_BURNING_PYRO);

    //Steam-bound clothing color
    SetEntityColor(CalcMercColor());

    //On-spawn invul
    SetPropInt(self, "m_bForcedSkin", 1);
    SetPropInt(self, "m_nForcedSkin", 5);
    if (GetRoundState() == 3)
        self.AddCustomAttribute("no_attack", 1, 5.2);
    self.AddCondEx(TF_COND_INVULNERABLE, GetRoundState() == 3 ? 5 : 2, null);
    RunWithDelay( GetRoundState() == 3 ? 5 : 2, function() {
        SetPropInt(self, "m_nForcedSkin", 4);
    });

    //Spawn sound
    if (GetRoundState() >= 4)
        EmitSoundEx({
            sound_name = "respawn.wav",
            volume = 1,
            sound_level = 150,
            flags = 1,
            channel = 0,
            entity = self,
            speaker_entity = self
        });
});

function RefreshCharacter(keepCurrentHP = true)
{
    if (!IsValidRound())
        return;
    local oldHP = self.GetHealth();
    local weaponTT = GetWeaponTypeFromEntity(self.GetActiveWeapon());
    local playerClass = safeget(weaponTT, "playerclass", null);
    if (playerClass)
        self.SetPlayerClass(playerClass);
    else
        playerClass = self.GetPlayerClass();
    self.AddCustomAttribute("move speed bonus",
        1.04 / MERC_MOVESPEED_MULTIPLIER[playerClass],
        -1);

    self.SetCustomModelWithClassAnimations(API_GetString("merc_model"));

    self.SetHealth(keepCurrentHP ? oldHP : 200);
    self.SetMaxHealth(200);
    self.RemoveCustomAttribute("max health additive penalty");
    self.AddCustomAttribute("max health additive penalty", MERC_HP_OFFSETS[playerClass] + 50, -1);

    RefreshModels();
    RunWithDelay(0.1, RefreshModels);
}

OnSelfGameEvent("player_death", -100, function()
{
    if (!IsValidRound())
        return;
    EmitSoundEx({
        sound_name = vo_death_sounds[RandomInt(0, vo_death_sounds.len() - 1)],
        entity = self,
        sound_level = 90,
    });
    RunWithDelay(0.1, function(victim) {
        SetPropBool(self, "m_bAllowInstantSpawn", false);
    });
    /*RunWithDelay(3, function(victim) {
        if (IsValidPlayer(victim))
            victim.ForceRespawn();
    }, self, main_script);*/
});

OnSelfGameEvent("player_death_post", 0, function()
{
    if (!IsValidRound())
        return;
    local ragdoll = GetPropEntity(self, "m_hRagdoll");
    SetPropBool(ragdoll, "m_bGib", false);
    SetPropInt(ragdoll, "m_clrRender", 4444444);
});

local moveKeys = IN_ATTACK | IN_ATTACK2 | IN_FORWARD | IN_BACK | IN_MOVELEFT | IN_MOVERIGHT;

AddTimer(-1, 0, function()
{
    if (GetPropInt(self, "m_nButtons") & moveKeys && GetRoundState() > 3)
    {
        self.RemoveCond(TF_COND_INVULNERABLE);
        SetPropInt(self, "m_nForcedSkin", 4);
    }
})

function SetEntityColor(rgb)
{
    local color = rgb[0] | (rgb[1] << 8) | (rgb[2] << 16);
    SetPropInt(self, "m_clrRender", color);
}

function CalcMercColor()
{
    local steamId = GetPropString(self, "m_szNetworkIDString");
    local indexStr = steamId.len() > 5 ? steamId.slice(5) : "4444444";
    local index = indexStr.tointeger();
    return [index % 256, (index >> 8) % 256, (index >> 16) % 256, 255];
}